/*
 * SWE Ereigniskalender - Version 2
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax: +49-341-49067-15
 * E-Mail: info@bitctrl.de
 */

package de.bsvrz.vew.ereigniskal.data;

import java.time.LocalDateTime;
import java.util.LinkedHashSet;
import java.util.Set;

import de.bsvrz.dav.daf.main.config.SystemObject;

/**
 * Repräsentiert eine Anfrage an den Ereigniskalender.
 * 
 * Die Anfrage wird im Normalbetrieb aus der entsprechenden Attributgruppe des
 * Datenverteilers konstruiert.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class KalenderAnfrage {

    private LocalDateTime anfangsZeitpunkt = LocalDateTime.now();
    private LocalDateTime endZeitpunkt = LocalDateTime.now();
    private Set<SystemObject> raeumlicheGueltigkeit = new LinkedHashSet<>();
    private EreignisTypenOption ereignisTypOption = EreignisTypenOption.Alle;
    private Set<SystemObject> ereignisTypen = new LinkedHashSet<>();

    /**
     * liefert den Anfangszeitpunkt des Anfragezeitraums
     * 
     * @return den Zeitpunkt
     */
    public LocalDateTime getAnfangsZeitpunkt() {
        return anfangsZeitpunkt;
    }

    /**
     * liefert den Endzeitpunkt des Anfragezeitraums
     * 
     * Liegt der Endzeitpunkt vor dem Anfangszeitpunkt, wird der
     * Anfangszeitpunkt geliefert.
     * 
     * @return den Zeitpunkt
     */
    public LocalDateTime getEndZeitpunkt() {
        if (endZeitpunkt.isBefore(anfangsZeitpunkt)) {
            endZeitpunkt = anfangsZeitpunkt;
        }

        return endZeitpunkt;
    }

    /**
     * prüft, ob das übergebenen Ereignis bei der Beantwortung der Anfrage
     * berücksichtigt werden soll.
     * 
     * Ereignisse können über die in der Anfrage festgelegten Einschränkungen
     * der räumlichen Gültigkeit und über den Ereignistyp von der Betrachtung
     * ausgeschlossen werden.
     * 
     * @param ereignis
     *            das Ereignis
     * @return true, wenn das Ereignis berücksichtigt werden soll
     */
    public boolean verwendeEreignis(Ereignis ereignis) {

        if (!ereignis.getRaeumlicheGueltigkeit().istGueltigFuer(raeumlicheGueltigkeit)) {
            return false;
        }

        switch (ereignisTypOption) {
        case Nicht:
            return !ereignisTypen.contains(ereignis.getEreignisTyp());
        case Nur:
            return ereignisTypen.contains(ereignis.getEreignisTyp());
        case Alle:
            return true;
        default:
            throw new IllegalStateException("Unbekannte Ereignistyp-Option: " + ereignisTypOption);
        }
    }

    /**
     * setzt den Anfangszeitpunkt des Abfragezeitraums.
     * 
     * @param anfangsZeitpunkt
     *            der Zeitpunkt
     */
    protected void setAnfangsZeitpunkt(LocalDateTime anfangsZeitpunkt) {
        this.anfangsZeitpunkt = anfangsZeitpunkt;
    }

    /**
     * setzt den Endzeitpunkt des Abfragezeitraums.
     * 
     * @param endZeitpunkt
     *            der Zeitpunkt
     */
    protected void setEndZeitpunkt(LocalDateTime endZeitpunkt) {

        this.endZeitpunkt = endZeitpunkt;
    }

    /**
     * liefert die in der Anfrage definierten Objekte, die die räumliche
     * Gültigkeit definieren.
     * 
     * @return die Menge der Objekte
     */
    protected Set<SystemObject> getRaeumlicheGueltigkeit() {
        return raeumlicheGueltigkeit;
    }

    /**
     * liefert die Menge der Ereignistypen, die die Auswahl der Ereignisse für
     * die Beantwortung der Anfrage einschränken können.
     * 
     * @return die Typen
     */
    protected Set<SystemObject> getEreignisTypen() {
        return ereignisTypen;
    }

    /**
     * setzt die Option zur Auswahl der für die Beantwortung der Anfrage zu
     * verwendenden Ereignisse.
     * 
     * @param ereignisTypOption
     *            die Option
     */
    protected void setEreignisTypOption(EreignisTypenOption ereignisTypOption) {
        this.ereignisTypOption = ereignisTypOption;
    }

    /**
     * fügt der Anfrage ein Objekt zur Beschreibung der räumlichen Gültigkeit
     * hinzu.
     * 
     * @param systemObject
     *            das Objekt
     * @return die Kalenderanfrage selbst
     */
    public KalenderAnfrage withNetzBestandteil(SystemObject systemObject) {
        getRaeumlicheGueltigkeit().add(systemObject);
        return this;
    }
}
