/*
 * SWE Ereigniskalender - Version 2
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax: +49-341-49067-15
 * E-Mail: info@bitctrl.de
 */

package de.bsvrz.vew.ereigniskal.data;

import java.time.Instant;
import java.time.LocalDateTime;
import java.time.ZoneId;

import de.bsvrz.vew.syskal.KalenderEintrag;
import de.bsvrz.vew.syskal.SystemKalender;
import de.bsvrz.vew.syskal.SystemkalenderGueltigkeit;
import de.bsvrz.vew.syskal.ZustandsWechsel;

/**
 * Beschreibung der zeitlichen Gültigkeit eines Ereignisses.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class ZeitlicheGueltigkeit {

    private LocalDateTime beginn;
    private LocalDateTime ende;
    private KalenderEintrag kalenderEintrag;

    /**
     * erzeugt eine neue Definition der zeitlichen Gültigkeit für den
     * übergebenen Zeitraum.
     * 
     * @param von
     *            der Anfangszeitpunkt in Millisekunden der lokalen Zeit seit
     *            1.1.1970
     * @param bis
     *            der Endzeitpunkt in Millisekunden der lokalen Zeit seit
     *            1.1.1970
     * @return die neue Instanz
     */
    public static ZeitlicheGueltigkeit of(long von, long bis) {
        return of(LocalDateTime.ofInstant(Instant.ofEpochMilli(von), ZoneId.systemDefault()),
                LocalDateTime.ofInstant(Instant.ofEpochMilli(bis), ZoneId.systemDefault()));
    }

    /**
     * erzeugt eine neue Definition der zeitlichen Gültigkeit für den
     * übergebenen Zeitraum.
     * 
     * @param von
     *            der Anfangszeitpunkt
     * @param bis
     *            der Endzeitpunkt
     * @return die neue Instanz
     */
    public static ZeitlicheGueltigkeit of(LocalDateTime von, LocalDateTime bis) {

        ZeitlicheGueltigkeit result = new ZeitlicheGueltigkeit();

        result.beginn = von;
        result.ende = bis;

        return result;
    }

    /**
     * erzeugt eine neue Definition der zeitlichen Gültigkeit unter Verwendung
     * des übergebenen Systemkalendereintrags.
     * 
     * @param kalenderEintrag
     *            der Kalendereintrag
     * @return die neue Instanz
     */
    public static ZeitlicheGueltigkeit of(KalenderEintrag kalenderEintrag) {

        ZeitlicheGueltigkeit result = new ZeitlicheGueltigkeit();
        result.kalenderEintrag = kalenderEintrag;
        return result;
    }

    private ZeitlicheGueltigkeit() {
        // Instanzen werden nur per Factory-Methode erzeugt
    }

    /**
     * ermittelt, ob die zeitliche Gültigkeit zum übergebenen Zeitpunkt gegeben
     * ist.
     * 
     * @param zeitPunkt
     *            der Zeitpunkt
     * @return die Gültigkeit
     */
    public boolean isGueltig(LocalDateTime zeitPunkt) {

        if (kalenderEintrag != null) {
            return kalenderEintrag.isGueltig(zeitPunkt);
        }

        if (beginn != null && ende != null) {
            return (!zeitPunkt.isBefore(beginn)) && zeitPunkt.isBefore(ende);
        }

        return false;
    }

    /**
     * berechnet die Systemkalendergueltigkeit für den übergebenen Zeitpunkt.
     * 
     * Die Systemkalendergültigkeit enthält Informationen zu den Zeitpunkten des
     * aktuellen und des erwarteten Zustandswechsel.
     * 
     * @param zeitPunkt
     *            der Zeitpunkt
     * @return die Gültigkeit
     */
    public SystemkalenderGueltigkeit getGueltigkeit(LocalDateTime zeitPunkt) {
        if (kalenderEintrag != null) {
            return kalenderEintrag.getZeitlicheGueltigkeit(zeitPunkt);
        }

        if (beginn != null && ende != null) {
            if (zeitPunkt.isBefore(beginn)) {
                return SystemkalenderGueltigkeit.ungueltig(SystemKalender.MIN_DATETIME, beginn);
            }
            if (zeitPunkt.isBefore(ende)) {
                return SystemkalenderGueltigkeit.of(ZustandsWechsel.of(beginn, true), ZustandsWechsel.of(ende, false));
            }
            return SystemkalenderGueltigkeit.of(ZustandsWechsel.aufUngueltig(ende),
                    ZustandsWechsel.aufUngueltig(SystemKalender.MAX_DATETIME));
        }

        return SystemkalenderGueltigkeit.NICHT_GUELTIG;
    }

    /**
     * berechnet die Systemkalendergueltigkeit vor dem übergebenen Zeitpunkt.
     * 
     * Die Systemkalendergültigkeit enthält Informationen zu den Zeitpunkten des
     * aktuellen und des vorigen Zustandswechsel.
     * 
     * @param zeitPunkt
     *            der Zeitpunkt
     * @return die Gültigkeit
     */
    public SystemkalenderGueltigkeit getVorigenGueltigkeitsBereich(LocalDateTime zeitPunkt) {
        if (kalenderEintrag != null) {
            SystemkalenderGueltigkeit gueltigkeitVor = kalenderEintrag
                    .getZeitlicheGueltigkeitVor(zeitPunkt);
            while (!gueltigkeitVor.isZeitlichGueltig()
                    && gueltigkeitVor.getErsterWechsel().getZeitPunkt().isAfter(SystemKalender.MIN_DATETIME)) {
                gueltigkeitVor = kalenderEintrag
                        .getZeitlicheGueltigkeitVor(gueltigkeitVor.getErsterWechsel().getZeitPunkt());
            }
            return gueltigkeitVor;
        }

        if (beginn != null && ende != null) {
            if (zeitPunkt.isBefore(ende)) {
                return SystemkalenderGueltigkeit.NICHT_GUELTIG;
            }
            return SystemkalenderGueltigkeit.of(ZustandsWechsel.of(beginn, true), ZustandsWechsel.of(ende, false));
        }

        return SystemkalenderGueltigkeit.NICHT_GUELTIG;
    }

    /**
     * berechnet die folgende Systemkalendergueltigkeit nach dem übergebenen
     * Zeitpunkt.
     * 
     * Die Systemkalendergültigkeit enthält Informationen zu den Zeitpunkten der
     * nachfolgenden Zustandswechsel.
     * 
     * @param zeitPunkt
     *            der Zeitpunkt
     * @return die Gültigkeit
     */
    public SystemkalenderGueltigkeit getNaechstenGueltigkeitsBereich(LocalDateTime zeitPunkt) {

        if (kalenderEintrag != null) {
            SystemkalenderGueltigkeit gueltigkeitNach = kalenderEintrag
                    .getZeitlicheGueltigkeit(zeitPunkt);

            while (!gueltigkeitNach.isZeitlichGueltig()
                    && gueltigkeitNach.getNaechsterWechsel().getZeitPunkt().isBefore(SystemKalender.MAX_DATETIME)) {
                gueltigkeitNach = getGueltigkeit(gueltigkeitNach.getNaechsterWechsel().getZeitPunkt());
            }
            return gueltigkeitNach;
        }

        if (beginn != null && ende != null) {
            if (!zeitPunkt.isAfter(beginn)) {
                return SystemkalenderGueltigkeit.of(ZustandsWechsel.of(beginn, true), ZustandsWechsel.of(ende, false));
            }
        }

        return SystemkalenderGueltigkeit.NICHT_GUELTIG;
    }
}