/*
 * SWE Ereigniskalender - Version 2
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax: +49-341-49067-15
 * E-Mail: info@bitctrl.de
 */

package de.bsvrz.vew.ereigniskal.module;

import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.time.LocalDateTime;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.ClientReceiverInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.ReceiveOptions;
import de.bsvrz.dav.daf.main.ReceiverRole;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;
import de.bsvrz.vew.ereigniskal.data.Ereignis;
import de.bsvrz.vew.ereigniskal.data.RaeumlicheGueltigkeit;
import de.bsvrz.vew.ereigniskal.data.VerkehrlicheGueltigkeit;
import de.bsvrz.vew.ereigniskal.data.ZeitlicheGueltigkeit;
import de.bsvrz.vew.syskal.KalenderEintrag;
import de.bsvrz.vew.syskal.SystemKalenderEintrag;
import de.bsvrz.vew.syskal.SystemKalenderException;

/**
 * Abbildung eines Ereignisses mit den für die Bereitstellung von Daten
 * erforderlichen Datenverteiler-Kopplungen.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class DavEreignis extends Ereignis implements ClientReceiverInterface {

    private static final Debug LOGGER = Debug.getLogger();

    private final DataDescription ereignisParameterDesc;

    private EreignisVerwaltung verwaltung;

    private EreignisPublisher ereignisPublisher;

    private PropertyChangeListener kalenderEintragsListener = e -> kalenderEintragAenderung(e);

    private SystemKalenderEintrag systemKalenderEintrag;

    DavEreignis(EreignisVerwaltung verwaltung, SystemObject systemObject) {
        super(systemObject.getName());

        setSystemObject(systemObject);

        setEreignisTyp(systemObject
                .getConfigurationData(systemObject.getDataModel().getAttributeGroup("atg.ereignisEigenschaften"))
                .getReferenceValue("EreignisTypReferenz").getSystemObject());

        ereignisParameterDesc = new DataDescription(
                systemObject.getDataModel().getAttributeGroup("atg.ereignisParameter"),
                systemObject.getDataModel().getAspect("asp.parameterSoll"));
        ereignisPublisher = new EreignisPublisher(verwaltung, this);

        this.verwaltung = verwaltung;
    }

    private void kalenderEintragAenderung(PropertyChangeEvent e) {
        Object newValue = e.getNewValue();
        if (newValue instanceof KalenderEintrag) {
            setZeitlicheGueltigkeit(ZeitlicheGueltigkeit.of((KalenderEintrag) newValue));
            publiziereStatus();
        }

    }

    void connect(ClientDavInterface dav) {
        dav.subscribeReceiver(this, getSystemObject(), ereignisParameterDesc, ReceiveOptions.normal(),
                ReceiverRole.receiver());
        ereignisPublisher.connect();
    }

    void disconnect(ClientDavInterface dav) {
        dav.unsubscribeReceiver(this, getSystemObject(), ereignisParameterDesc);
        ereignisPublisher.disconnect();
    }

    private void updateEreignis(Data data) {

        if (systemKalenderEintrag != null) {
            systemKalenderEintrag.removeKalenderEintragChangeListener(kalenderEintragsListener);
        }

        setRaeumlicheGueltigkeit(RaeumlicheGueltigkeit.from(data));

        SystemObject syskalRef = data.getReferenceValue("SystemKalenderEintragReferenz").getSystemObject();
        if (syskalRef == null) {
            setZeitlicheGueltigkeit(
                    ZeitlicheGueltigkeit.of(data.getTimeValue("BeginnZeitlicheGültigkeit").getMillis(),
                            data.getTimeValue("EndeZeitlicheGültigkeit").getMillis()));
        } else {
            try {
                systemKalenderEintrag = verwaltung.getSystemKalender().getEintrag(syskalRef);
                setZeitlicheGueltigkeit(ZeitlicheGueltigkeit.of(systemKalenderEintrag.getKalenderEintrag()));
                systemKalenderEintrag.addKalenderEintragChangeListener(kalenderEintragsListener);
            } catch (SystemKalenderException e) {
                LOGGER.warning(getName() +
                        ": der referenzierte Systemkalendereintrag konnte nicht ermittelt werden,"
                        + " es erfolgt keine Änderungsüberwachung!: "
                        + e.getLocalizedMessage());
            }
        }

        setVerkehrlicheGueltigkeit(VerkehrlicheGueltigkeit.from(data));

        publiziereStatus();
    }

    void publiziereStatus() {

        EreignisGueltigkeit gueltigkeit = berechneEreignisGueltigkeit(LocalDateTime.now());
        if (!verwaltung.getEreignisWechselUeberwacher().gueltigKeitGeaendert(this, gueltigkeit)) {
            return;
        }

        try {
            ereignisPublisher.publiziereGueltigkeit(gueltigkeit);
            verwaltung.getEreignisWechselUeberwacher().setGueltigkeit(this, gueltigkeit);
        } catch (DataNotSubscribedException | SendSubscriptionNotConfirmed e) {
            LOGGER.warning(getName() + " Ereignisstatus konnte nicht publiziert werden: " + e.getLocalizedMessage());
        }
    }

    @Override
    public void update(ResultData[] results) {
        for (ResultData result : results) {
            if (result.hasData()) {
                if (result.getDataDescription().getAttributeGroup().equals(ereignisParameterDesc.getAttributeGroup())) {
                    updateEreignis(result.getData());
                }
            }
        }
    }

}
