/*
 * SWE Ereigniskalender - Version 2
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax: +49-341-49067-15
 * E-Mail: info@bitctrl.de
 */

package de.bsvrz.vew.ereigniskal.module;

import de.bsvrz.dav.daf.main.ClientSenderInterface;
import de.bsvrz.dav.daf.main.Data;
import de.bsvrz.dav.daf.main.DataDescription;
import de.bsvrz.dav.daf.main.DataNotSubscribedException;
import de.bsvrz.dav.daf.main.OneSubscriptionPerSendData;
import de.bsvrz.dav.daf.main.ResultData;
import de.bsvrz.dav.daf.main.SendSubscriptionNotConfirmed;
import de.bsvrz.dav.daf.main.SenderRole;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.sys.funclib.debug.Debug;

/**
 * Publisher zur Veröffentilchung des aktuellen Status eines Ereignisses über
 * den Datenverteiler.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class EreignisPublisher implements ClientSenderInterface {

    private static final Debug LOGGER = Debug.getLogger();

    private DataDescription ereignisStatusDesc;
    private EreignisVerwaltung verwaltung;
    private DavEreignis davEreignis;

    /**
     * erzeugt eine Instanz eines Publishers für das in der übergebenen
     * Verwaltung verwaltete Ereignis.
     * 
     * @param verwaltung
     *            die Verwaltung
     * @param davEreignis
     *            das Ereignis
     */
    public EreignisPublisher(EreignisVerwaltung verwaltung, DavEreignis davEreignis) {
        this.verwaltung = verwaltung;
        this.davEreignis = davEreignis;
        ereignisStatusDesc = new DataDescription(
                davEreignis.getSystemObject().getDataModel().getAttributeGroup("atg.ereignis"),
                davEreignis.getSystemObject().getDataModel().getAspect("asp.zustand"));

    }

    @Override
    public void dataRequest(SystemObject object, DataDescription dataDescription, byte state) {
        // wird nicht ausgwertet, weil eine Quellenanmeldung vorliegt
    }

    @Override
    public boolean isRequestSupported(SystemObject object, DataDescription dataDescription) {
        return false;
    }

    /** meldet das Ereignis beim Datenverteiler an. */
    public void connect() {
        try {
            verwaltung.getDav().subscribeSender(this, davEreignis.getSystemObject(), ereignisStatusDesc,
                    SenderRole.source());
        } catch (OneSubscriptionPerSendData e) {
            LOGGER.warning(davEreignis.getName() + ": Unerwarteter Fehler bei der Datenverteiler-Kommunikation: "
                    + e.getLocalizedMessage());
        }
    }

    /** meldet das Ereignis beim Datenverteiler ab. */
    public void disconnect() {
        verwaltung.getDav().unsubscribeSender(this, davEreignis.getSystemObject(), ereignisStatusDesc);
    }

    /**
     * versendet den übergebenen Zustand des Ereignisses
     * 
     * @param gueltigkeit
     *            die zu versendende Gültigkeit
     * @throws DataNotSubscribedException
     *             Fehler beim Versand der Daten über den Datenverteiler
     * @throws SendSubscriptionNotConfirmed
     *             Sendefreigabe wurde vom Datenverteiler noch nicht erteilt
     */
    public void publiziereGueltigkeit(EreignisGueltigkeit gueltigkeit)
            throws DataNotSubscribedException, SendSubscriptionNotConfirmed {
        Data data = verwaltung.getDav().createData(ereignisStatusDesc.getAttributeGroup());
        data.getUnscaledValue("zeitlichGültig").setText(gueltigkeit.isZeitlichGueltig() ? "Ja" : "Nein");
        data.getUnscaledValue("verkehrlichGültig").setText(gueltigkeit.isVerkehrlichGueltig() ? "Ja" : "Nein");
        data.getUnscaledValue("AttributÄnderung").setText("Ja");
        data.getTimeValue("Zeitpunkt").setMillis(gueltigkeit.getVoraussichtlicheAenderung());
        verwaltung.getDav()
                .sendData(
                        new ResultData(davEreignis.getSystemObject(), ereignisStatusDesc, verwaltung.getDav().getTime(),
                                data));
    }
}
