/*
 * SWE Ereigniskalender - Version 2
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 * 
 * Contact Information:
 * BitCtrl Systems GmbH
 * Weißenfelser Straße 67
 * 04229 Leipzig
 * Phone: +49-341-49067-0
 * Fax: +49-341-49067-15
 * E-Mail: info@bitctrl.de
 */

package de.bsvrz.vew.ereigniskal.module;

import java.util.List;
import java.util.Map;
import java.util.concurrent.ConcurrentHashMap;

import de.bsvrz.dav.daf.main.ClientDavInterface;
import de.bsvrz.dav.daf.main.config.ConfigurationObject;
import de.bsvrz.dav.daf.main.config.MutableSet;
import de.bsvrz.dav.daf.main.config.MutableSetChangeListener;
import de.bsvrz.dav.daf.main.config.SystemObject;
import de.bsvrz.vew.ereigniskal.data.KalenderAnfrage;
import de.bsvrz.vew.syskal.SystemKalender;

/**
 * Verwaltung der Ereignisse eine Kalenderobjekts des Datenverteilermodells.
 * 
 * @author BitCtrl Systems GmbH, Uwe Peuker
 */
public class EreignisVerwaltung implements MutableSetChangeListener {

    private Map<SystemObject, DavEreignis> davEreignisse = new ConcurrentHashMap<>();
    private EreignisProvider provider = new EreignisProvider();

    private ClientDavInterface dav;

    private SystemKalender systemKalender;
    private EreignisWechselUeberwacher ereignisWechselUeberwacher;

    /**
     * erzeugt eine neue INstanz der Verwaltung für die übergebenen
     * Datenverteilerverbindung und das angegebenen Kalenderobjekt.
     * 
     * @param dav
     *            die Datenverteilerverbindung
     * @param kalenderObject
     *            der Kalender
     */
    public EreignisVerwaltung(ClientDavInterface dav, ConfigurationObject kalenderObject) {

        systemKalender = new SystemKalender(dav, kalenderObject);
        ereignisWechselUeberwacher = new EreignisWechselUeberwacher();

        this.dav = dav;

        MutableSet ereignisSet = kalenderObject.getMutableSet("Ereignisse");
        ereignisSet.addChangeListener(this);
        ereignisSet.getElements().stream().filter(so -> so.isValid() && so.isOfType("typ.ereignis"))
                .forEach(so -> addEreignis(so));
    }

    void addEreignis(SystemObject so) {

        if (!so.isValid()) {
            return;
        }

        DavEreignis ereignis = new DavEreignis(this, so);
        DavEreignis altesEreignis = davEreignisse.put(so, ereignis);
        provider.addEreignis(ereignis);
        if (altesEreignis != null) {
            altesEreignis.disconnect(dav);
        }

        ereignis.connect(dav);
    }

    void removeEreignis(SystemObject so) {

        DavEreignis ereignis = davEreignisse.remove(so);
        provider.removeEreignis(so.getName());
        if (ereignis != null) {
            ereignis.disconnect(dav);
        }
    }

    /**
     * liefert die verwendete Datenverteilerverbindung
     * 
     * @return die Datenverteilerverbindung
     */
    public ClientDavInterface getDav() {
        return dav;
    }

    /**
     * ermittelt die Liste der Zustandswechsel entsprechend der übergebenen
     * {@link KalenderAnfrage}.
     * 
     * @param anfrage
     *            die Anfrage
     * @return die Liste der Zustandswechsel
     */
    public List<EreignisZustandsWechsel> getEreignisZustandsWechsel(KalenderAnfrage anfrage) {
        return provider.getEreignisZustandsWechsel(anfrage);
    }

    @Override
    public void update(MutableSet set, SystemObject[] addedObjects, SystemObject[] removedObjects) {

        for (SystemObject so : removedObjects) {
            removeEreignis(so);
        }
        for (SystemObject so : addedObjects) {
            addEreignis(so);
        }
    }

    SystemKalender getSystemKalender() {
        return systemKalender;
    }

    EreignisWechselUeberwacher getEreignisWechselUeberwacher() {
        return ereignisWechselUeberwacher;
    }

}
